/* src/chanserv.c - Diverses commandes sur le module chanserv
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Gay@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: chanserv.c,v 1.86 2004/12/02 22:37:08 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "cs_cmds.h"
#include "outils.h"
#include "chanserv.h"
#include "ban.h"
#include "add_info.h"
#include "del_info.h"
#include "fichiers.h"

inline void show_accessn(anAccess *acces, anUser *user, aNick *num)
{
	csntc(num, GetReply(num, L_ACUSER), user->nick, acces->level);
	csntc(num, "\2Options:\2%s", GetAccessOptions(acces));
	if(acces->lastseen == 1) csntc(num, GetReply(num, L_ACONCHAN));
	else csntc(num, GetReply(num, L_ACLASTSEEN), get_time(num, acces->lastseen));
	if(acces->info) csntc(num, "\2Infoline:\2 %s", acces->info);
}

/*
 * show_access <chan> <recherche>
 */
int show_access(aNick *nick, aChan *chan, int parc, char **parv)
{
	register aLink *lp = chan->access;
	char *nick2 = parv[2], comp = '\0';
	int i = 0, autoop, protect, suspend, autovoice, all, wait, l = 0, wild = 0;

	if(HasMode(chan, C_MSECRET | C_MPRIVATE) /* salon +s ou +p */
		&& (!nick->user || !GetAccessIbyUserI(nick->user, chan)) /* non logu ou n'a pas d'accs */
		&& !IsAnAdmin(nick->user) && !GetJoinIbyC(nick, chan)) /* ET ni admin, ni prsent */
			return csntc(nick, GetReply(nick, L_YOUNOACCESSON), chan->chan);

	if(*nick2 == '>' || *nick2 == '<' || *nick2 == '=')
	{
		comp = *nick2++;
		if(!*nick2 || !is_num(nick2))
			return csntc(nick, "La valeur du niveau doit tre numrique derrire <,> ou =");
		l = atoi(nick2);
	}
	else wild = HasWildCard(nick2);

	autoop = getoption("-autoop", parv, parc, 3, -1);
	autovoice = getoption("-autovoice", parv, parc, 3, -1);
	protect = getoption("-protect", parv, parc, 3, -1);
	suspend = getoption("-suspend", parv, parc, 3, -1);
	wait = getoption("-wait", parv, parc, 3, -1);
	all = getoption("-all", parv, parc, 3, -1) && (IsAnAdmin(nick->user)
		|| chan->owner->user == nick->user || ChanLevelbyUserI(nick->user, chan) >= 450) ? 1 : 0;

	csntc(nick, GetReply(nick, L_ACCESSLIST), chan->chan);

	for(;lp;lp = lp->next)
	{
		register anAccess *a = lp->value.a;

		if(((wait && AWait(a)) || (!wait && !AWait(a)))
			&& ((comp != '\0' && ((comp == '>' && a->level > l) /* recherche */
			          || (comp == '<' && a->level < l)	/* via level */
			          || (comp == '=' && l == a->level)))
			  || (*nick2 == '*' && !nick2[1]) /* recherche sur '*' */
			  || (wild /* ou sur un vrai mask */
			  		&& !match(nick2, a->user->nick))
			  || !strcasecmp(nick2, a->user->nick)) /* ou access exact */
			&& (!autoop || AOp(a)) /* ET match des options Aop/Avoice/Suspend etc */
			&& (!autovoice || AVoice(a))
			&& (!protect || AProtect(a))
			&& (!suspend || ASuspend(a)))
		{
			if(++i > MAXACCESSMATCHES && !all) continue;
			show_accessn(a, a->user, nick);
			csntc(nick, "-");
		}
	}

	if(i > MAXACCESSMATCHES && !all)
		csntc(nick, GetReply(nick, L_EXCESSMATCHES), i, MAXACCESSMATCHES);
	else csntc(nick, GetReply(nick, L_TOTALFOUND), i, PLUR(i), PLUR(i));
	return 0;
}

/*
 * add_user <salon> <user> <level>
 */
int add_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	char *username = parv[2], *salon = parv[1];
#ifdef USE_MEMOSERV
	char memo[MEMOLEN + 1];
#endif
	int lvl;

	if(!(u = ParseNickOrUser(nick, username))) return 0;

	if(!is_num(parv[3]) || (lvl = atoi(parv[3])) < 1 || lvl >= OWNERLEVEL)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if(lvl >= ChanLevelbyUserI(nick->user, chaninfo) && !IsAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

	if(u->flag & U_PREJECT)
		return csntc(nick, GetReply(nick, L_XREFUSEACCESS), u->nick);

	for(a = u->accesshead;a && a->c != chaninfo;a = a->next);
	if(a)
	{
		if(AWait(a)) csntc(nick, GetReply(nick, L_ALREADYAWAIT), u->nick);
		else csntc(nick, GetReply(nick, L_ALREADYACCESS), u->nick, salon);
		return 0;
	}

	if(u->flag & U_PACCEPT)
	{
		add_access(u, salon, lvl, (lvl >= 400) ? (A_OP | A_PROTECT) : 0, CurrentTS);
		csntc(nick, GetReply(nick, L_XHASACCESS), u->nick, lvl, salon);
		if(u->n) csntc(u->n, GetReply(u->n, L_YOUHAVEACCESS), nick->nick, lvl, salon);
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, GetReply(nick, L_YOUHAVEACCESS), nick->nick, lvl, salon);
			add_memo(u, nick->user->nick, CurrentTS, memo, 0);
		}
#endif
	}
	else
	{
		add_access(u, salon, lvl, lvl >= 400 ? A_OP|A_PROTECT|A_WAITACCESS : A_WAITACCESS, CurrentTS);
		csntc(nick, GetReply(nick, L_XHASAWAIT), u->nick, salon);

		if(u->n)
		{
			csntc(u->n, GetReply(u->n, L_YOUHAVEAWAIT1), nick->nick, lvl, salon);
			csntc(u->n, GetReply(u->n, L_YOUHAVEAWAIT2), cs.nick, RealCmd("myaccess"),
				salon, cs.nick, RealCmd("myaccess"), salon);
		}
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, GetReply(nick, L_YOUHAVEAWAIT1), nick->nick, lvl, salon);
			add_memo(u, nick->user->nick, CurrentTS, memo, 0);
		}
#endif
	}
	return write_anuser(u);
}

/*
 * del_user <salon> <user>
 */
int del_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	const char *salon = parv[1];

	if(!(u = ParseNickOrUser(nick, parv[2]))) return 0;

	for(a = u->accesshead;a && a->c != chaninfo;a = a->next);
	if(!a) return csntc(nick, GetReply(nick, L_XNOACCESSON), u->nick, salon);

	if(ChanLevelbyUserI(nick->user, chaninfo) <= a->level && !IsAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

	if(a->level == OWNERLEVEL) return csntc(nick, GetReply(nick, L_XISOWNER), u->nick);

	del_access(u, chaninfo);
	write_anuser(u);

	csntc(nick, GetReply(nick, L_OKDELETED), u->nick);
	return 1;
}

int kick(aNick *nick, aChan *chan, int parc, char **parv)
{
	const char *c = parv[1], *cible = parv[2];
	aNick *targ;

	if(!HasWildCard(cible))
	{	/*la cible est un nick unique .. */

		if(!strcasecmp(cs.nick, cible)) return csntc(nick, "Non!");

		if(!(targ = GetMemberIbyNick(chan, cible)))
			return csntc(nick, GetReply(nick, L_NOTONCHAN), cible, chan->chan);

		if(!check_protect(nick, targ, chan)) return 0;

		cskick(c, targ->numeric, "(\2$\2) $", nick->user->nick,
			(parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150));
	}
	else
	{	/* kick sur un mask*/
		const char *r = (parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150);
		aLink *lp = chan->members;

		for(;lp;lp = lp->next)
		{
			targ = lp->value.j->nick;

			if(nick != targ && !match(cible, GetNUHbyNick(targ, 0)) && check_protect(nick, targ, chan))
				cskick(c, targ->numeric, "(\2$\2) $", nick->user->nick, r);
		}
	}
	return 1;
}

int mode(aNick *nick, aChan *chan, int parc, char **parv)
{
	int a = 0;
	char *modes = parv[2], *c = strchr(modes, 'o');

	if(c || (c = strchr(modes, 'b')) || (c = strchr(modes, 'v'))
	 || (!IsOper(nick) && (c = strchr(modes, 'O'))))
		return csntc(nick, GetReply(nick, L_CANTCHANGEMODE), *c);

	if(chan->cml && !IsAdmin(nick->user) && ChanLevelbyUserI(nick->user, chan) < chan->cml)
		return csntc(nick, GetReply(nick, L_CMLIS), chan->cml);

	if(strchr(modes, 'k')) a++;
	if(strchr(modes, 'l')) a++;

	csmode(chan, 0, "$ $ $", modes, (a && parc > 2) ? parv[3] : "",
		(a > 1 && parc > 3) ? parv[4] : "");
	return 1;
}

int topic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, GetReply(nick, L_LOCKTOPIC), parv[1]);

	cstopic(chaninfo, parv2msg(parc, parv, 2, TOPICLEN));
	write_achan(chaninfo);
	return 1;
}

int rdeftopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, GetReply(nick, L_LOCKTOPIC), parv[1]);

	if(*chaninfo->deftopic) cstopic(chaninfo, chaninfo->deftopic);
	return 1;
}

int rdefmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, 0, "-$", GetCModes(chaninfo->mode));
	if(chaninfo->defmodes.modes) csmode(chaninfo, 0, "+$", GetCModes(chaninfo->defmodes));
	return 1;
}

int info(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anAccess *acces = GetAccessIbyUserI(nick->user, chaninfo);

	if(!acces) return 0;

	if(parc < 2) return csntc(nick, GetReply(nick, L_INFOLINEORNONE));

	if(!strcasecmp(parv[2], "none"))
	{
		if(acces->info) free(acces->info), acces->info = NULL;
		csntc(nick, GetReply(nick, L_OKDELETED), chaninfo->chan);
	}
	else
	{
		str_dup(&acces->info, parv2msg(parc, parv, 2, 199));
		csntc(nick, GetReply(nick, L_INFOCHANGED), chaninfo->chan, acces->info);
	}
	return write_anuser(nick->user);
}

int invite(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *inv = nick;

	if(parc > 1 && strcasecmp(parv[2], nick->nick))
	{
		if(!IsAdmin(nick->user)) return csntc(nick, GetReply(nick, L_CANTINVELSE));
		else if(!(inv = getnickbynick(parv[2]))) return csntc(nick, GetReply(nick, L_NOSUCHNICK), parv[2]);
	}

	if(GetJoinIbyC(inv, chaninfo))
		return csntc(nick, GetReply(nick, L_ALREADYONCHAN), inv->nick, chaninfo->chan);

	if(is_ban(inv, chaninfo, NULL))
		return csntc(nick, GetReply(nick, L_XISBANCANTINV), inv->nick, chaninfo->chan);

	putserv("%s " TOKEN_INVITE " %s :%s", cs.num, inv->nick, chaninfo->chan);
	putserv("%s " TOKEN_WALLCHOPS " %s :Invitation de %s",	cs.num,	chaninfo->chan, inv->nick);
	return 1;
}

int clearmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, 0, "-$", GetCModes(chaninfo->mode));
	return 1;
}

/*
 * see_alist #
 */
int see_alist(aNick *nick, aChan *chan, int parc, char **parv)
{
	aLink *lp = chan->access;
	char buf[400] = {0}, tmp[2*NICKLEN+9];
	int i = 0, j = 0, c = 0;

	for(;lp;lp = lp->next)
	{
		anAccess *a = lp->value.a;
		if(a->user->n && !AWait(a) && !ASuspend(a))
		{
			int tmps = i;
			i += (j = sprintf(tmp, "%s@%s[\002%d\2] ", a->user->n->nick, a->user->nick, a->level));
			if(i >= sizeof buf)
			{
				csntc(nick, c++ ? "%s" : "Liste des identifis: %s", buf);
				i = j;
				strcpy(buf, tmp);
			}
			else strcpy(&buf[tmps], tmp);
		}
	}
	if(*buf) csntc(nick, !c ? "Liste des identifis: %s" : "%s", buf);
	else if(!c) csntc(nick, GetReply(nick, L_NOINFOAVAILABLE));
	return 1;
}

int admin_say(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :%s", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

int admin_do(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :\1ACTION %s\1", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

