/* src/fichiers.c - Lecture/criture des donnes
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: config.c,v 1.20 2004/11/26 18:44:26 romexzf Exp $
 */

#include "main.h"
#include "config.h"
#include "outils.h"
#include "debug.h"
#include <errno.h>
#ifdef TESTBOT
#include <arpa/inet.h>
#include "web2cs.h"
#endif

int kill_interval = 25;
int ignoretime = 600;
int maxlastseen = 1000000;
int chanmaxidle = 7200;
char *quit_msg = NULL;
char *pasdeperm = NULL;
char *network = NULL;
char *mailprog = NULL;
char HIDDEN_HOST[HOSTLEN + 1];

static struct tab {
	const char *item;
	int id;
	const char *description;
} 	conftab[] = {
	{"csbot", 		2, "configuration des infos du Channel Service"},
	{"uplink", 		3, "configuration des infos ncessaire au link (ip, port)"},
	{"myserver", 	4, "configuration des infos du serveur du CS"},
	{"misc_conf", 	5, "configuration des diverses options (On The Fly!) des Services"},
#ifdef USE_WELCOMESERV
	{"wsbot", 		6, "configuration des infos du Welcome Service"},
#endif
#ifdef TESTBOT
	{"web2cs", 		7, "configuration du web2cs"},
#endif
	{"lang", 		8, "configuration du multilanguage"},
#ifdef SQLLOG
	{"sql_log", 	9, "configuration du serveur MySQL de log"},
#endif
	{"", 0, ""}
},
	uplink[] = {
	{"ip", 0, "IP du HUB auquel le serveur doit se connecter"},
	{"pass", 0, "Pass du link"},
	{"port", 0, "Port du HUB auquel le serveur doit se connecter"},
	{"", 0, ""}
},
	csbot[] = {
	{"nick", 0, "Pseudo du CS"},
	{"ident", 0, "Ident du CS"},
	{"host", 0, "Host du CS"},
	{"realname", 0, "Real Name du CS"},
	{"modes", 0, "UserModes que doit porter le CS  la connexion (+ok normalement)"},
	{"chan", 0, "Salon de log o est envoye l'activit du CS"},
	{"", 0, ""}
},
	myserv[] = {
	{"server", 0, "Nom littral du serveur"},
	{"infos", 0, "Infos du serveur, visible dans le /links"},
	{"numeric", 0, "Numeric P10 du serveur (en chiffre)"},
	{"network", 0, "Nom du rseau"},
	{"quit_msg", 0, "Message envoy lors de la maintenance des services"},
	{"", 0, ""}
},
	misc_conf[] = {
	{"commandchar", 0, "caractre prefixant les commandes salons"},
	{"mailprog", 0, "path vers Sendmail"},
	{"hidden_host", 0, "fin de l'host lors du changement de l'host en <login>.<HIDDEN_HOST>"},
	{"pas_de_perm", 0, "message envoy  un user qui utilise une commande  laquelle il n'a pas access"},
	{"help_chan", 0, "salon d'aide"},
	{"kill_time", 0,"Intervalle qu'attend le robot avant d'enforcer les protections de pseudo"},
	{"change_nick", 0, "active/desactive le changement de nick au lieu d'un KILL"},
	{"maxlastseen", 0, "lastseen maximal avant la suppression de l'username"},
	{"adminexempt", 0, "exemption des admins pour la protection anti flood"},
	{"kill_for_flood", 0, "si actif, les users qui flood seront kill"},
	{"ignoretime", 0, "nombre de secondes qu'ignore le robot"},
	{"host_hidding", 0, "active le changement de l'host des users en <login>.<HIDDEN_HOST> (HIDDEN_HOST est paramtrable)"},
	{"xmode", 0, "Si actif, le masquage de l'host ne se fera pas lors de l'identification mais lors de la mise du +x"},
	{"chanmaxidle", 0, "Temps avant que le CS ne quitte un salon vide"},
	{"", 0, ""}
},
	lang[] = {
	{"lang", 0, "Nom du fichier langage (sans le .lang"},
	{"", 0, ""}
};

#ifdef USE_WELCOMESERV
static struct tab wsbot[] = {
	{"nick", 0, "Pseudo du WS"},
	{"ident", 0, "Ident du WS"},
	{"host", 0, "Host du WS"},
	{"realname", 0, "Real Name du WS"},
	{"modes", 0, "UserModes que doit porter le WS  la connexion (+dk normalement)"},
	{"privmsg", 0, "Permet de choisir si le welcome est envoy en PRIVMSG (1) ou en NOTICE (0)"},
	{"", 0, ""}
};
#endif

#ifdef TESTBOT
static struct tab web2cs_conf[] = {
	{"port", 0, "Port du serveur web2cs"},
	{"allow_host", 0, "Host autorise  se connecter au serveur web2cs"},
	{"pass", 0, "Pass que dlivrera le php pour se connecter (non crypt)"},
	{"", 0, ""}
};
#endif

#ifdef SQLLOG
static struct tab sql_log[] = {
	{"ip", 0, "IP du serveur MySQL"},
	{"port", 0, "Port du serveur MySQL"},
	{"pass", 0, "Pass du serveur MySQL"},
	{"user", 0, "Login du serveur MySQL"},
	{"db", 0, "Nom de la DB"},
	{"", 0, ""}
};
#endif

static struct tab *tabptr[] = { NULL, NULL, csbot, uplink, myserv,  misc_conf,
#ifdef USE_WELCOMESERV
	wsbot,
#else
	NULL,
#endif
#ifdef TESTBOT
	web2cs_conf,
#else
	NULL,
#endif
	lang,
#ifdef SQLLOG
	sql_log,
#endif
	NULL
};

static int get_item(struct tab *tab, const char *item)
{
	int i = 0;
	for(;tab[i].item[0] != '\0'; i++) if(!strcasecmp(tab[i].item, item)) return i;
	return -1;
}

static int check_missing_item(struct tab *tab, int *real_tab)
{
	int i = 0, miss = 0;
	*real_tab = 0;
	for(;tab[i].item[0] != '\0';i++)
		if(tab[i].id != 1 && !(tab[i].id & CONF_READ))
		{
			printf("conf: item '%s' manquant (%s)\n", tab[i].item, tab[i].description);
			miss++;
		}
	return miss;
}

static char *conf_parse(char *ptr, struct tab *tab, int current_tab, int item)
{
	ptr = strtok(NULL, "=\r\n");
	while(ptr && *ptr && (*ptr == ' ' || *ptr == '\t')) ptr++;
	if(!ptr || !*ptr)
	{
		printf("conf: tab '%s', item '%s' sans valeur.\n", conftab[current_tab].item, tab[item].item);
		return NULL;
	}
	tab[item].id = 1;
	return ptr;
}

static int readconf(FILE *fp)
{
	int line = 0, current_tab = 0, tmp = 0, real_tab = 0;
	char buf[512] = {0};
	struct tab *real_tabptr = NULL;

#ifdef TESTBOT
	w2c_hallow *hnew = w2c_hallowhead, *hnext;
	/* sinon lors d'un /z rehash les hosts mises seront AJOUTES*/
	for(;hnew;free(hnew), hnew = hnext) hnext = hnew->next;

	w2c_hallowhead = NULL;
#endif

	while(fgets(buf, sizeof buf, fp))
	{
		char *ptr = buf;
		while(*ptr == '\t' || *ptr == ' ') ptr++; /* on saute les espaces/tabs*/
		if(*ptr == '#' || *ptr == '\r' || *ptr == '\n' || !*ptr) continue; /* commentaire OU ligne vide*/
		ptr = strtok(ptr, " ");

		line++;
		if(!real_tab) /* no tab selected!*/
		{
			if((current_tab = get_item(conftab, ptr)) == -1) printf("conf line: %d, tab inconnue: %s\n", line, ptr);
			else
			{
				real_tab = (conftab[current_tab].id & ~CONF_READ); /* index de la tab*/
				conftab[current_tab].id |= CONF_READ; /* tab trouve -- marquons le */
				real_tabptr = tabptr[real_tab];
			}
			continue;
		}
		else
		{
			if(EndOfTab(ptr))
			{
				if(check_missing_item(real_tabptr, &real_tab)) return -1;/* end of /tab*/
				continue;
			}
			if((tmp = get_item(real_tabptr, ptr)) == -1) {
				printf("conf: line %d, tab %s, item inconnu: %s\n", line, conftab[current_tab].item, ptr);
				continue;
			}
			else if(!(ptr = conf_parse(ptr, real_tabptr, current_tab, tmp))) return -1;
		}

		switch(real_tab)
		{
			case 3: /* uplink */
			switch(tmp)
			{
				case 0:
					if(!is_ip(ptr))
					{
						printf("L'IP %s semble invalide. (line %d)\n", ptr, line);
						return -1;
					}
					strcpy(bot.ip, ptr);
					break;
				case 1:
					Strncpy(bot.pass, ptr, 21);
					break;
				case 2:
					if(!is_num(ptr) || (tmp = atoi(ptr)) <= 0 || tmp > 65535)
					{
						printf("Port de link invalide: (line %d)\n", line);
						return -1;
					}
					bot.port = tmp;
					break;
			}
			break;

			case 2: /* csbot */
			switch(tmp)
			{
				case 0:
					if(!IsValidNick(ptr))
					{
						printf("Erreur: line %d: le nick du CS '%s' comporte des caractres non conformes\n", line, ptr);
						return -1;
					}
					Strncpy(cs.nick, ptr, NICKLEN);
					if(strlen(ptr) > NICKLEN) printf("Nick du CS trop long, tronqu en %s (line %d)\n", cs.nick, line);
					break;
				case 1:
					Strncpy(cs.ident, ptr, USERLEN);
					if(strlen(ptr) > USERLEN) printf("Ident du CS trop longue, tronque en %s (line %d)\n", cs.ident, line);
					break;
				case 2:
					Strncpy(cs.host, ptr, HOSTLEN);
					if(strlen(ptr) > HOSTLEN) printf("Host du CS trop longue, tronque en %s (line %d)\n", cs.host, line);
					break;
				case 3:
					Strncpy(cs.name, ptr, REALEN);
					if(strlen(ptr) > REALEN) printf("RealName du CS trop long, tronqu en %s (line %d)\n", cs.name, line);
					break;
				case 4:
					Strncpy(cs.mode, ptr, 9);
					break;
				case 5:
					if(*ptr != '#')
					{
						printf("Format de salon invalide '%c' (line %d)\n", *ptr, line);
						return -1;
					}
					Strncpy(bot.pchan, ptr, REGCHANLEN);
					break;
			}
			break;

			case 4: /* myserver */
			switch(tmp)
			{
				case 0:
					if(!strchr(ptr, '.'))
					{
						printf("Le nom du Serveur des services semble invalide. (line %d)\n",line);
						return -1;
					}
					Strncpy(bot.server, ptr, HOSTLEN);
					break;
				case 1:
					Strncpy(bot.name, ptr, REALEN);
					break;
				case 2:
					if(!is_num(ptr) || atoi(ptr) >  MAXNUM)
					{
						printf("Numeric non valide %s (Devrait etre entre 0 et 64/4096) (line %d)\n", ptr, line);
						return -1;
					}
					make_servnum(atoi(ptr), bot.servnum);
					break;
				case 3:
				    if(strchr(ptr, ' '))
				    {
						printf("Le nom du rseau doit tre compos d'un seul mot (ex: Inter-System.net) (line %d)\n", line);
						return -1;
					}
					str_dup(&network, ptr);
					break;
				case 4:
					str_dup(&quit_msg, ptr);
					break;
			}
			break;

			case 5: /* misc_conf*/
			switch(tmp)
			{
				case 0:
					bot.cara = *ptr;
					break;
				case 1:
					if(!strcasecmp(ptr, "nomail")) ConfFlag |= CF_NOMAIL;
					else str_dup(&mailprog, ptr);
					break;
				case 2:
					Strncpy(HIDDEN_HOST, ptr, HOSTLEN);
					break;
				case 3:
					str_dup(&pasdeperm, ptr);
					break;
				case 4:
					if(*ptr != '#')
					{
						printf("Format de salon invalide '%c' (line %d)\n", *ptr, line);
						return -1;
					}
					Strncpy(bot.chan, ptr, REGCHANLEN);
					break;
				case 5:
					if((kill_interval = convert_duration(ptr)) <= 0)
					{
						printf("Le temps d'attente pour les KILLs doit tre un nombre (line %d)\n", line);
						return -1;
					}
					break;
				case 6:
					if(atoi(ptr)) ConfFlag |= CF_CHANGENICK;
					else ConfFlag &= ~CF_CHANGENICK;
					break;
				case 7:
					if((maxlastseen = convert_duration(ptr)) <= 0)
					{
						printf("Le temps maximum avant la purge doit tre un nombre (line %d)\n", line);
						return -1;
					}
					break;
				case 8:
					if(!is_num(ptr))
					{
						printf("Admin Exempt pour le flood doit tre  0 ou 1 (line %d)\n", line);
						return -1;
					}
					if(atoi(ptr)) ConfFlag |= CF_ADMINEXEMPT;
					else ConfFlag &= ~CF_ADMINEXEMPT;
					break;
				case 9:
					if(!is_num(ptr))
					{
						printf("Le Kill pour flood doit tre  1 ou 0 (line %d)\n", line);
						return -1;
					}
					if(atoi(ptr)) ConfFlag |= CF_KILLFORFLOOD;
					else ConfFlag &= ~CF_KILLFORFLOOD;
					break;
				case 10:
					if((ignoretime = convert_duration(ptr)) < 0)
					{
						printf("Le temps d'ignore doit tre un nombre (line %d)\n", line);
						return -1;
					}
					break;
				case 11:
					if(atoi(ptr)) ConfFlag |= CF_HOSTHIDING;
					else ConfFlag &= ~CF_HOSTHIDING;
					break;
				case 12:
					if(atoi(ptr)) ConfFlag |= CF_XMODE;
					else ConfFlag &= ~CF_XMODE;
					break;
				case 13:
					if((chanmaxidle = convert_duration(ptr)) <= 0)
					{
						printf("Le temps d'attente avant le dpart des salons vides est invalide (line %d)\n", line);
						return -1;
					}
					break;
			}

			break;

#ifdef USE_WELCOMESERV
			case 6: /* wsbot */
			switch(tmp)
			{
				case 0:
					if(!IsValidNick(ptr))
					{
						printf("Erreur: line %d: le nick du WS '%s' comporte des caractres non conformes\n", line, ptr);
						return -1;
					}
					Strncpy(ws.nick, ptr, NICKLEN);
					if(strlen(ptr) > NICKLEN) printf("Nick du WS trop long, tronqu en %s (line %d)\n", ws.nick, line);
					break;
				case 1:
					Strncpy(ws.ident, ptr, USERLEN);
					if(strlen(ptr) > USERLEN) printf("Ident du WS trop longue, tronque en %s (line %d)\n", ws.ident, line);
					break;
				case 2:
					Strncpy(ws.host, ptr, HOSTLEN);
					if(strlen(ptr) > HOSTLEN) printf("Host du WS trop longue, tronque en %s (line %d)\n", ws.host, line);
					break;
				case 3:
					Strncpy(ws.name, ptr, REALEN);
					if(strlen(ptr) > REALEN) printf("RealName du WS trop long, tronqu en %s (line %d)\n", ws.name, line);
					break;
				case 4:
					Strncpy(ws.mode, ptr, 10);
					break;
				case 5:
					if(atoi(ptr)) ConfFlag |= CF_PRIVWELCOME;
					else ConfFlag &= ~CF_PRIVWELCOME;
			}
			break;
#endif
#ifdef TESTBOT
			case 7: /* web2cs */
			switch(tmp)
			{
				case 0:
					if(!is_num(ptr) || (tmp = atoi(ptr)) < 1 || tmp > 65535)
					{
						printf("Erreur: line %d: le port du Web2Cs (%d) est invalide: il doit tre entre 1 et 65535 inclus.\n", line, tmp);
						return -1;
  					}
					bot.w2c_port = tmp;
					break;
				case 1:/*allow_host*/
					if(!is_ip(ptr))
					{
						printf("Erreur: line %d: la valeur (%s) doit tre une IP\n", line, ptr);
						return -1;
					}
					hnew = malloc(sizeof *hnew);
					hnew->host = inet_addr(ptr); /* ip */
					hnew->next = w2c_hallowhead; /* add trust to list */
					w2c_hallowhead = hnew;
					break;
				case 2: /*pass*/
					Strncpy(bot.w2c_pass, ptr, sizeof bot.w2c_pass -1);
					break;
			}
			break;
#endif
			case 8: /* multi lang*/
				if(lang_add(ptr) < 0) printf("Erreur: line %d: Impossible de charger le langage %s\n", line, ptr);
				break;

#ifdef SQLLOG
			case 9: /* sql log*/
			switch(tmp)
			{
				case 0:/*db host*/
					str_dup(&bot.sql_host, ptr);
					break;
				case 1:
					if(!is_num(ptr) || (tmp = atoi(ptr)) < 1 || tmp > 65535)
					{
						printf("Erreur: line %d: le port du serveur MySQL (%d) est invalide: il doit tre entre 1 et 65535 inclus.\n", line, tmp);
						return -1;
					}
					bot.sql_port = tmp;
					break;
				case 2: /*pass*/
					Strncpy(bot.sql_pass, ptr, 15);
					break;
				case 3: /* user*/
					Strncpy(bot.sql_user, ptr, 30);
					break;
				case 4: /*db*/
					Strncpy(bot.sql_db, ptr, 30);
					break;
			}
			break;
#endif
		}
	}

	if(real_tab)
	{
		printf("Conf: erreur, parsage incomplet, missing '}', last tab : %s\n", conftab[current_tab-1].item);
		return -1;
	}
	if(check_missing_item(conftab, &real_tab)) return -1;
	return 1;
}

int load_config(const char *file)
{
	FILE *fp = fopen(file, "r");
	int error = 0;
	if(!fp)
	{
		Debug(W_TTY|W_MAX, "conf: Impossible d'ouvrir le fichier de conf (%s)", strerror(errno));
		return -1;
	}
	error = readconf(fp);
	fclose(fp);
	return error;
}
