/* src/hash_nick.c - gestion des hash nick
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: hash_nick.c,v 1.16 2004/11/09 15:55:45 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "admin_manage.h"
#include "hash.h"
#include "del_info.h"
#include "debug.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include <ctype.h>
#ifdef HAVE_TRACK
#include "track.h"
#endif

static const unsigned char convert2n[] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  52,53,54,55,56,57,58,59,60,61, 0, 0, 0, 0, 0, 0,
   0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
  15,16,17,18,19,20,21,22,23,24,25,62, 0,63, 0, 0,
   0,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
  41,42,43,44,45,46,47,48,49,50,51, 0, 0, 0, 0, 0
};

static inline unsigned int do_hashn(const char *nick)
{
	unsigned int checksum = 0;
	while(*nick) checksum += (checksum << 3) + tolower(*nick++);
	return checksum & (NICKHASHSIZE-1);
}


unsigned int base64toint(const char *s)
{
    unsigned int i = 0;
    while (*s) i = (i << 6) + convert2n[(unsigned char) *s++];
    return i;
}

static int hash_delnick(aNick *nick)
{
	unsigned int hash = do_hashn(nick->nick);
	aNick *tmp = nick_tab[hash];

	if(tmp == nick) nick_tab[hash] = nick->next;
	else
	{
		for(;tmp && tmp->next != nick;tmp = tmp->next);
		if(tmp) tmp->next = nick->next;
		else Debug(W_WARN|W_MAX, "H_del_nick %s non trouv  l'offset %u ?!", nick->nick, hash);
	}
	return 0;
}

int switch_nick(aNick *nick, const char *newnick)
{
	unsigned int hash = do_hashn(newnick);

	hash_delnick(nick);
	Strncpy(nick->nick, newnick, NICKLEN);
	nick->next = nick_tab[hash];
	nick_tab[hash] = nick;
	return 0;
}

aNick *num2nickinfo(const char *num)
{
#ifdef N2K
	unsigned int servindex = convert2n[(unsigned char)num[1]] + 64 * convert2n[(unsigned char)num[0]],
	numindex = (convert2n[(unsigned char)num[4]] + 64 * convert2n[(unsigned char)num[3]] + 4096 * convert2n[(unsigned char)num[2]]) & serv_tab[servindex]->smask;
#else
	unsigned int servindex = convert2n[(unsigned char)num[0]],
	numindex = (convert2n[(unsigned char)num[2]] + 64 * convert2n[(unsigned char)num[1]]) & serv_tab[servindex]->smask;
#endif

	return num_tab[servindex][numindex];
}

aNick *getnickbynick(const char *nick)
{
	unsigned int hash = do_hashn(nick);
	register aNick *tmp = nick_tab[hash];
	for(;tmp && strcasecmp(nick, tmp->nick);tmp = tmp->next);
	return tmp;
}

aNick *add_nickinfo(const char *nick, const char *user, const char *host, const char *base64,
			const char *num, aServer *server, const char *name, time_t ttmco, const char *umode)
{
	aNick *new = calloc(1, sizeof *new);
#ifdef N2K
	unsigned int servindex = convert2n[(unsigned char)num[1]] + 64 * convert2n[(unsigned char)num[0]],
	numindex = (convert2n[(unsigned char)num[4]] + 64 * convert2n[(unsigned char)num[3]] + 4096 * convert2n[(unsigned char)num[2]]) & server->smask,
#else
	unsigned int servindex = convert2n[(unsigned char)num[0]],
	numindex = (convert2n[(unsigned char)num[2]] + 64 * convert2n[(unsigned char)num[1]]) & server->smask,
#endif
	hash = do_hashn(nick);

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_nickinfo, malloc a chou pour aNick %s", nick);
		return NULL;
	}

	if(num_tab[servindex][numindex])
	{
		aNick *nptr = num_tab[servindex][numindex];
		Debug(W_DESYNCH|W_WARN, "add_nickinfo, Offset pour %s(%s@%s) [%s] dj pris par %s(%s@%s) [%s]",
			nick, user, host, num, nptr->nick, nptr->ident, nptr->host, nptr->numeric);
		free(new);
		return NULL;
	}

	num_tab[servindex][numindex] = new;/* table des pointeurs num -> struct*/

	Strncpy(new->nick, nick, NICKLEN);
	Strncpy(new->ident, user, USERLEN);
	Strncpy(new->host, host, HOSTLEN);
	Strncpy(new->base64, base64, 7);
	strcpy(new->numeric, num);
	new->serveur = server;
	Strncpy(new->name, name, REALEN);
	new->ttmco = ttmco;
	new->floodtime = CurrentTS;
	new->floodcount = 0;

	if(umode) new->flag = parse_umode(0, umode);
#ifdef HAVE_CRYPTHOST /* cryptage de l'host en md2 ou md5 suivant certains critres*/
	if(new->flag & N_SERVICE) strcpy(new->crypt, host);
	else hostprot(host, new->crypt);
#endif

	new->next = nick_tab[hash];/* hash table des nick -> struct*/
	nick_tab[hash] = new;
	return new;
}

void del_nickinfo(const char *num, const char *event)
{
	aJoin *join, *chanp = NULL;
	aNick *nick;
#ifdef N2K
	unsigned int servindex = convert2n[(unsigned char)num[1]] + 64 * convert2n[(unsigned char)num[0]],
	numindex = convert2n[(unsigned char)num[4]] + 64 * convert2n[(unsigned char)num[3]] + 4096 * convert2n[(unsigned char)num[2]];
#else
	unsigned int servindex = convert2n[(unsigned char)num[0]],
	numindex = convert2n[(unsigned char)num[2]] + 64 * convert2n[(unsigned char)num[1]];
#endif

	if(!serv_tab[servindex])
	{
		Debug(W_DESYNCH|W_WARN, "del_nickinfo(%s), del %s sur un serveur inconnu!", event, num);
		return;
	}
	numindex &= serv_tab[servindex]->smask;/* on rduit par le max du serveur (attribution des nums..)*/

	if(!num_tab[servindex][numindex])
	{
		Debug(W_DESYNCH|W_WARN, "del_nickinfo(%s), del %s membre inconnu", event, num);
		return;
	}

	nick = num_tab[servindex][numindex];

	join = nick->joinhead;
	while(join)
	{
		chanp = join;
		if(join->chan) /* c'est un chan reg (conomie d'un getchaninfo..)*/
		{
			anAccess *a; /* mise  jour du lastseen */
			if(nick->user && (a = GetAccessIbyUserI(nick->user, join->chan)))
				a->lastseen = CurrentTS;
			del_link(join->chan, nick);
		}
		join = join->next;
		free(chanp);
	}

	if(nick->user)	/* sauvergarde du lastlogin */
	{
#ifdef HAVE_TRACK
		struct track *track = trackhead, *tt = NULL;
		for(;track;track = tt)
		{
			tt = track->next;
			if(nick->user == track->tracked)
				csntc(track->tracker, "[\2Track\2] %s (User: %s) vient de quitter (%s).",
					nick->nick, track->tracked->nick, event);
			if(nick == track->tracker) del_track(track->tracked);
		}
#endif

		str_dup(&nick->user->lastlogin, GetNUHbyNick(nick, 0));
		write_anuser(nick->user);
		if(IsAdmin(nick->user)) adm_active_del(nick);
		nick->user->n = NULL;
	}
	del_killinfo(nick, NULL); /* purge de tous ses kills */

	num_tab[servindex][numindex] = NULL;/* tab des num purge */
	hash_delnick(nick);/* swap du nick dans la hash table */
	free(nick);
}

int purge_nickandserv(void)
{
	int i = 0, j;
	for(;i < MAXNUM;i++)
		if(num_tab[i])
		{
			for(j = 0;j < serv_tab[i]->maxusers;j++)
				if(num_tab[i][j]) del_nickinfo(num_tab[i][j]->numeric, "clearing");

			free(serv_tab[i]);
			free(num_tab[i]);
			num_tab[i] = NULL;
			serv_tab[i] = NULL;
		}

	return 0;
}
