/* src/web2cs-engine.c - Moteur du Web2CS
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: web2cs-engine.c,v 1.61 2004/11/19 20:36:28 romexzf Exp $
 */

/* NEW CS LIVE*/

#include "main.h"
#ifdef TESTBOT
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/time.h>
#include <errno.h>
#include "web2cs.h"
#include "hash.h"
#include "outils.h"
#include "debug.h"
#include "ban.h"
#include "config.h"
#include "cs_cmds.h"
#include "del_info.h"
#include "fichiers.h"
#include "add_info.h"
#include "sql_log.h"
#include "crypt.h"

#define MAXPARAM 16
#define W2CDEBUG
/*#define W2CDEBUGALL*/

/* dear global vars */

int bind_sock = 0;
fd_set global_fd_set;
int highsock = 0;

static WClient myWClient[MAXWEBCON+1];
static time_t w2c_lastcheck = 0;

w2c_hallow *w2c_hallowhead = NULL;

#ifdef W2CDEBUG

#	ifdef W2CDEBUGALL
/* Web2CS debug function, basicaly logs everything. */
static int w2cdebug(const char *fmt, ...)
{
	FILE *fd = fopen("logs/w2c.log", "a");
	va_list vl;

	if(!fd) return Debug(W_WARN, "Error while opening w2c log file : %s", strerror(errno));

	va_start(vl, fmt);
	fputs(get_time(NULL, CurrentTS), fd); /* TimeStamp */
	fputc(' ', fd);
	vfprintf(fd, fmt, vl);
	fputc('\n', fd);
	fclose(fd);

	va_end(vl);
	return 0;
}

#		define DEBUGF(x) w2cdebug x
#	else
#		define DEBUGF(x)
#	endif
#endif
/*** Socks ***/

/* send current client's buffer */
static inline int w2c_dequeue(WClient *cl)
{
	send(cl->fd, cl->QBuf, cl->buflen, 0);
	WDelFlush(cl);
	cl->buflen = 0;
	return 0;
}

/* format reply, queue the reply in send buffer and flush it if needed */
static int w2c_sendrpl(WClient *cl, const char *pattern, ...)
{
	static char buf[512];
	va_list vl;
	int len;

	va_start(vl, pattern);
	len = vsnprintf(buf, sizeof buf - 2, pattern, vl); /* format */
	if(len < 0) len = sizeof buf -2;

	DEBUGF(("sending [%s] to %s", buf, cl->ip));

	buf[len++] = '\n';
	buf[len] = 0;
	va_end(vl);

	bot.WEBtrafficUP += len;

	if(cl->buflen + len >= W2C_SENDSIZE) w2c_dequeue(cl); /* would overflow, flush it */
	strcpy(cl->QBuf + cl->buflen, buf); /* append reply to buffer */
	cl->buflen += len;
	if(WIsFlush(cl)) w2c_dequeue(cl); /* Is flush forced ? */

	return 0;
}

/* disconnect a client, allow a reason to be given */
static int w2c_exit_client(WClient *cl, const char *msg)
{
	if(msg && *msg) w2c_sendrpl(cl, msg);

	w2c_delclient(cl);
	return 0;
}

/* **** CMDS **** */

static int w2c_passwd(WClient *cl, int parc, char **parv)
{
	if(IsAuth(cl)) return 0; /* On devrait pas kill le sock?*/
	if(strcmp(parv[1], bot.w2c_pass))
		return w2c_exit_client(cl, "erreur Mauvais pass Web2CS");
	cl->flag |= W2C_AUTH;
	return 0;
}

static int w2c_login(WClient *cl, int parc, char **parv)
{
	anUser *user;

	if(cl->user) return w2c_exit_client(cl, "erreur Dj logu");

	if(!(user = getuserinfo(parv[1])))
		return w2c_exit_client(cl, "erreur Username inexistant");

	if(IsSuspend(user)) return w2c_exit_client(cl, "erreur Username suspendu");

	if(strcmp(user->passwd, parv[2]))
	{
		if(IsAdmin(user))
		{
			cswall("(Web) Login Admin chou sur \2%s\2 par \2%s\2", user->nick, parv[3]);
			putlog(LOG_FAUTH, "LOGIN %s (BadPass) par %s (Web)", user->nick, parv[3]);
		}
		return w2c_exit_client(cl, "erreur Mauvais pass");
	}

	user->lastseen = CurrentTS; /* login stuff : lastseen, lastlogin*/

	if(!user->lastlogin || strcmp(parv[3], user->lastlogin + 4))
	{
		char host[20] = "web@"; /* IPv4 + 'web@' + \0 */
		Strncpy(host + 4, parv[3], sizeof host - 5); /* wanna overflow me? */

		if(UOubli(user)) DelUOubli(user);
		str_dup(&user->lastlogin, host);

#ifdef SQLLOG
	 	sql_query(SQL_QINSERTU, "('%s', %T, 'login', '%s')", user->nick, CurrentTS, host);
#endif
	}

	cl->user = user;
	return 0;
}

#define SHOW_TOPIC 		0x01
#define SHOW_RIGHT 		0x02
#define SHOW_USERS 		0x04
#define SHOW_DEFMODES 	0x08
#define SHOW_OWNER 		0x10
#define SHOW_BANS 		0x20
#define SHOW_ACCESS 	0x40
#define SHOW_ALL 		0x80

static int w2c_showaccess(WClient *cl, aChan *chan, int flag)
{
	aLink *lp = chan->access;
	int count = 0;

	for(;lp;lp = lp->next)
	{
		anAccess *a = lp->value.a;
		w2c_sendrpl(cl, "access%d %s %d %lu %d :%s", ++count, a->user->nick, a->level,
			a->lastseen, a->flag, NONE(a->info));
	}

	return w2c_sendrpl(cl, "accesscount %d", count);
}

static int w2c_channel(WClient *cl, int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);

	if(!strcasecmp(parv[2], "info"))
	{
		int infos = 0, i;

		if(!chan) return w2c_exit_client(cl, "erreur Salon non reg");
		else if(parc < 4) return w2c_exit_client(cl, "OK Salon reg");

		if(IsAdmin(cl->user) || GetAccessIbyUserI(cl->user, chan)) infos |= SHOW_RIGHT;
		if(HasMode(chan, C_MSECRET|C_MPRIVATE) && !(infos & SHOW_RIGHT))
			return w2c_exit_client(cl, "erreur Ces infos ne vous sont disponibles.");

		for(i = 3;i < parc;++i)
		{
			if(!strcasecmp(parv[i], "all"))
			{
				infos |= SHOW_ALL;
				break;
			}
			else if(!strcasecmp(parv[i], "topic")) infos |= SHOW_TOPIC;
			else if(!strcasecmp(parv[i], "owner")) infos |= SHOW_OWNER;
			else if(!strcasecmp(parv[i], "users") && infos&SHOW_RIGHT) infos |= SHOW_USERS;
			else if(!strcasecmp(parv[i], "bans")) infos |= SHOW_BANS;
			else if(!strcasecmp(parv[i], "access")) infos |= SHOW_ACCESS;
		}

		if(!infos) return w2c_exit_client(cl, "OK Salon reg");/* aucune infos demandes*/

		if(infos & SHOW_OWNER)/* OK, construisons/envoyons les infos*/
			w2c_sendrpl(cl, "owner %s", chan->owner ? chan->owner->user->nick : "<Aucun>");
		if(infos & SHOW_ALL)
		{
			w2c_sendrpl(cl, "=owner=%s ct=%lu banlevel=%d cml=%d bantype=%d "
				"bantime=%ld options=%d liminc=%d limgrace=%d",
				chan->owner ? chan->owner->user->nick : "***", chan->creation_time,
				chan->banlevel,	chan->cml, chan->bantype, chan->bantime, chan->flag,
				chan->limit_inc, chan->limit_min);

			w2c_sendrpl(cl, "=modes=%u defmodes=%u limit=%d deflimit=%d key=%s defkey=%s",
				chan->mode.modes, chan->defmodes.modes, chan->mode.limit, chan->defmodes.limit,
				!HasMode(chan, C_MKEY) || infos & SHOW_RIGHT ? chan->mode.key : "***",
				!(chan->defmodes.modes&C_MKEY) || infos&SHOW_RIGHT ? chan->defmodes.key : "***");

			w2c_sendrpl(cl, "description %s", chan->description);
			if(*chan->deftopic) w2c_sendrpl(cl, "deftopic %s", chan->deftopic);
			if(*chan->welcome) w2c_sendrpl(cl, "welcome %s", chan->welcome);
			if(infos & SHOW_RIGHT && chan->motd) w2c_sendrpl(cl, "motd %s", chan->motd);
		}
		if(infos & (SHOW_ALL|SHOW_TOPIC) && *chan->topic)
			w2c_sendrpl(cl, "topic %s", chan->topic);
		if(infos & SHOW_ACCESS) w2c_showaccess(cl, chan, 0);
		if(infos & SHOW_BANS)
		{
			aBan *ban = chan->banhead;

			for(i = 0;ban;ban = ban->next)
				w2c_sendrpl(cl, "ban%u %s %s %lu %lu %d :%s", ++i, ban->mask,
					ban->de, ban->debut, ban->fin, ban->level, ban->raison);
			w2c_sendrpl(cl, "bancount %u", i);
		}
	}
	return w2c_exit_client(cl, "OK");
}

static int w2c_memolist(WClient *cl, anUser *user, int id)
{
	aMemo *m = user->memohead;
	int i = 0;

	for(;m;m = m->next)
	{
		if(++i == id) return w2c_sendrpl(cl, "memo %s %lu %d :%s",
							m->de, m->date, m->read, m->message);
		else if(!id) w2c_sendrpl(cl, "memo%d %s %lu %d :%s", i,
					m->de, m->date, m->read, m->message), m->read = 1;
	}
	w2c_sendrpl(cl, "memocount %d", i); /* only sent if id == 0 */
	return 0;
}

static int w2c_user(WClient *cl, int parc, char **parv)
{
	anUser *user;
	int adm = IsAdmin(cl->user) ? cl->user->level : 0;

	/* handle most of case whithout looping: Not Admin or query on himself */
	if(!adm || !strcasecmp(parv[1], cl->user->nick)) user = cl->user;
	else if(!(user = getuserinfo(parv[1]))) /* check admin query */
		return w2c_exit_client(cl, "erreur Username inconnu");

	if(!strcasecmp(parv[2], "info"))
	{
		anAccess *a;
		int count = 0;

		w2c_sendrpl(cl, "=user=%s mail=%s lastseen=%lu level=%d lastlogin=%s "
			"options=%d lang=%s", user->nick, user->mail, user->lastseen,
			user->level, user->lastlogin ? user->lastlogin : "<Indisponible>",
			user->flag, user->lang->langue);

		if(user == cl->user) w2c_memolist(cl, user, 0);

		for(a = user->accesshead;a;a = a->next)
			w2c_sendrpl(cl, "access%d %s %d %d %lu :%s", ++count, a->c->chan,
				a->level, a->flag, a->lastseen, NONE(a->info));
		w2c_sendrpl(cl, "accesscount %d", count);
	}
	else if(!strcasecmp(parv[2], "set"))
	{
		/*
		 * Set: can handle multi-options change on single line
		 */
		int i = 4, changes = 0;

		if(adm && user->level >= adm && user != cl->user)
			return w2c_exit_client(cl, "erreur Accs refus");

		for(;i < parc; i += 2) {
			char *opt = parv[i-1], *param = parv[i];

			if(!strcasecmp(opt, "mail")) {
				if(strlen(param) > MAILLEN || !IsValidMail(param) || !strcasecmp(user->mail, param))
					return w2c_exit_client(cl, "erreur invalid arg");

				if(GetUserIbyMail(param)) return w2c_exit_client(cl, "erreur mail dj utilis");

				strcpy(user->mail, param);
				++changes;
			} /* mail */

			else if(!strcasecmp(opt, "pass")) {
				if(strlen(param) < 7) return w2c_exit_client(cl, "erreur invalid arg");

				MD5pass(param, user->passwd);
#ifdef MD5TRANSITION
				SetUMD5(user); /* mark it as md5 */
#endif
				++changes;
			} /* password */

			else if(!strcasecmp(opt, "username")) {
				if(user->flag & U_ALREADYCHANGE && !adm)
					return w2c_exit_client(cl, "erreur username dj chang");

				if(strlen(param) > NICKLEN || !IsValidNick(param))
					return w2c_exit_client(cl, "erreur invalid arg");

				if(IsBadNick(param)) return w2c_exit_client(cl, "erreur User interdit");

				if(getuserinfo(param)) return w2c_exit_client(cl, "erreur user dj utilis");

				switch_user(user, param);
				++changes;
				if(!adm) user->flag |= U_ALREADYCHANGE;
			} /* username */

			else if(!strcasecmp(opt, "lang")) {
				Lang *lang = lang_isloaded(param);

				if(!lang)
					return w2c_exit_client(cl, "erreur language non disponible");

				user->lang = lang;
				++changes;
			} /* lang */

			else if(!strcasecmp(opt, "flag")) {
				int flag = atoi(param);

				if(!flag || flag == user->flag) continue; /* not a number, or no change */

#define U_ACCESSPOLICY (U_PREJECT|U_POKACCESS|U_PACCEPT)

				flag &= U_ALL; /* clear stranger flags */
				++changes;

				if(user->flag & U_ALREADYCHANGE && !(flag & U_ALREADYCHANGE))
					flag |= U_ALREADYCHANGE;

		/*		if((U_PKILL|U_PNICK) & flag == U_PKILL|U_PNICK
					|| !(U_ACCESSPOLICY & flag)	|| U_ACCESSPOLICY & ~(U_ACCESSPOLICY & flag))
					return w2c_exit_client(cl, "erreur flag ncessaire manquant");*/

				if((U_ACCESSPOLICY & user->flag) != (U_ACCESSPOLICY & flag))
				{
					anAccess *a, *at;
					if(flag & U_PREJECT) /* refuse all pending propositions */
						for(a = user->accesshead;a;a = at) {
							at = a->next;
							if(AWait(a)) del_access(user, a->c);
						}
					else if(flag & U_PACCEPT) /* accept all propositions */
						for(a = user->accesshead;a;a = a->next)
							if(AWait(a)) a->flag &= ~A_WAITACCESS;
				}
				user->flag = flag;
			} /* options */

		} /* for */

		/* no change performed */
		if(!changes) return w2c_exit_client(cl, "erreur Syntaxe incorrecte");

		/* at least one (doesn't check for bad syntax in extra changes) */
		w2c_sendrpl(cl, "=user=%s mail=%s level=%d options=%d lang=%s",
			user->nick, user->mail, user->level, user->flag, user->lang->langue);
		write_anuser(user);
	} /* set */
	else if(!strcasecmp(parv[2], "aaccept") || !strcasecmp(parv[2], "adrop"))
	{
		anAccess *a;
		aChan *c;
		int changes = 0, j = 2, del = 0;

		for(;j < parc;++j) {
			if(!strcasecmp(parv[j], "adrop")) del = 1;
			else if(!strcasecmp(parv[j], "aaccept")) del = 0;
			else for(a = user->accesshead, c = getchaninfo(parv[j]); c && a;a = a->next)
				if(a->c == c)
				{
					if(del) del_access(user, a->c), ++changes;
					else if(AWait(a)) a->flag &= ~A_WAITACCESS, ++changes;
					break;
				}
		}

		if(!changes) return w2c_exit_client(cl, "erreur Syntaxe incorrecte");
		w2c_sendrpl(cl, "performed %d", changes);
		write_anuser(user);
	} /* aaccess */

	return w2c_exit_client(cl, "OK");
}

static char *GetIP(const char *base64)
{
	struct in_addr addr2;

	addr2.s_addr = htonl(base64toint(base64));
	return inet_ntoa(addr2);
}

int w2c_register(WClient *cl, int parc, char **parv)
{
	anUser *user;
	aNick *nick;

	if(cl->user) return w2c_exit_client(cl, "erreur Vous possdez dj un username");

	if(strlen(parv[1]) > NICKLEN || strlen(parv[2]) > MAILLEN
		|| !IsValidNick(parv[1]) || !IsValidMail(parv[2]) || strlen(parv[3]) < 6)
		return w2c_exit_client(cl, "erreur arg invalide");

	if(IsBadNick(parv[1])) return w2c_exit_client(cl, "erreur username non enregistrable");
	if(getuserinfo(parv[1])) return w2c_exit_client(cl, "erreur user dj enregistr");

	if((nick = getnickbynick(parv[1])) && strcmp(GetIP(nick->base64), parv[4]))
		return w2c_exit_client(cl, "erreur ce pseudo est utilis par quelqu'un d'autre");

	if(GetUserIbyMail(parv[2])) return w2c_exit_client(cl, "erreur mail dj utilis");

	user = add_regnick(parv[1], MD5pass(parv[3], NULL), CurrentTS, 1, U_DEFAULT|U_FIRST, parv[2], -1);

	cl->user = user;
	user->lang = DefaultLang;

	append_file(USERS_FILE, user->nick);
	write_anuser(user);

#ifdef SQLLOG
	sql_query(SQL_QINSERTU, "('%s', %T, 'register', 'web@%s')", user->nick, CurrentTS, parv[4]);
#endif

	return w2c_exit_client(cl, "OK");
}

int w2c_regchan(WClient *cl, int parc, char **parv)
{
	aChan *chan;
	register aNick *n;
	register aJoin *j;
	const char *c = parv[1];
	int i = 1;

	for(;c[i] && i <= REGCHANLEN && c[i] != ',' && isascii(c[i]);++i);

	if(c[i] || *c != '#') return w2c_exit_client(cl, "erreur Syntaxe incorrecte");

	if(getchaninfo(c)) return w2c_exit_client(cl, "erreur AlreadyExist");
	if(IsBadChan(c)) return w2c_exit_client(cl, "erreur Salon interdit");

	if(CantRegChan(cl->user)) return w2c_exit_client(cl, "erreur CantRegChan");
	if(IsAnOwner(cl->user)) return w2c_exit_client(cl, "erreur AlreadyOwner");

	for(i = 0;i < NICKHASHSIZE;++i) for(n = nick_tab[i];n;n = n->next)
		for(j = n->joinhead;j;j = j->next) /* get chan users list */
			if(!strcasecmp(c, j->channel)) /* ok, user on chan */
				 return w2c_exit_client(cl, "erreur ChannelNotEmpty");

	chan = add_chan(c, parv2msg(parc, parv, 2, 200));
	add_access(cl->user, c, OWNERLEVEL, A_OP|A_PROTECT, CurrentTS);

	putlog(LOG_CHANS, "REGCHAN %s par %s@Web (%s)", c, cl->user->nick, chan->description);

	append_file(CHANS_FILE, c);
	write_achan(chan);
	write_anuser(cl->user);
	return 1;
}

int w2c_memo(WClient *cl, int parc, char **parv)
{
	aMemo *memo = cl->user->memohead;
	int i = 0, p = 0, found = 0;
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "READ"))
	{
		w2c_memolist(cl, cl->user, 0);
	}
	else if(!strcasecmp(cmd, "DEL"))
	{
		int todel[10], tmp = 0;
		aMemo *mnext, *mlast = NULL;

		if(parc < 2 || (strcasecmp(parv[2], "all")
			&& !(i = memo_parselist(parv[2], todel, ASIZE(todel)))))
			return w2c_exit_client(cl, "erreur arg invalide");

		for(;memo;memo = mnext)
		{
			mnext = memo->next;
			++p;
			if(!i) free(memo), found++;
			else if((tmp = memo_isinlist(todel, i, p)) != -1) /* need to be deleted */
			{
				found++; /* is in a list */
				if(mlast) mlast->next = mnext;
				else cl->user->memohead = mnext;
				free(memo);
				if(i == 1) break;
			}
			else mlast = memo;
		}

		if(!i && found) cl->user->memohead = NULL; /* all */

		w2c_memolist(cl, cl->user, 0);

		if(found) write_anuser(cl->user);
	}
	/*else if(!strcasecmp(cmd, "SEND"))
	{
		anUser *user;
		aNick *who;
		char *tmp;

		if(parc < 3) return w2c_exit_client(cl, "erreur erreur de syntaxe");

		if(!(user = getuserinfo(parv[2])))
			return w2c_exit_client(cl, "erreur Username inconnu");

		if(user == cl->user)
			return w2c_exit_client(cl, "erreur Ne vous envoyez pas de memo");

		if(UNoMemo(user) && !IsAnAdmin(cl->user))
			return w2c_exit_client(cl, "erreur Il n'accepte pas de memo");

		tmp = parv2msg(parc, parv, 3, MEMOLEN);

		if((who = user->n) && !IsAway(who))
			csntc(who, GetReply(who, L_MEMOFROM), cl->user->nick, cl->user->nick, tmp);

		for(memo = user->memohead;memo;memo = memo->next)
			if(!strcasecmp(memo->de, cl->user->nick)) i++;

		if(i >= MAXMEMOS && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_MAXMEMO), user->nick, MAXMEMOS);

		add_memo(user, cl->user->nick, CurrentTS, tmp, !!(who && !IsAway(who)));
		write_anuser(user);
	}*/

	return w2c_exit_client(cl, "OK");
}

/* ********* */

static struct w2c_commands {
   char cmd[CMDLEN + 1];
   int (*func) (WClient *, int, char **);
   int args;
} w2c_cmds[] = {
	{"pass",		w2c_passwd,		1},
	{"login",		w2c_login, 		3},
	{"user",		w2c_user,		2},
	{"register",	w2c_register, 	4},
	{"channel",		w2c_channel, 	2},
	{"memo", 		w2c_memo, 		1},
	{"regchan", 	w2c_regchan, 	2}
};

void w2c_setnonblocking(int sock)
{
	int opts = fcntl(sock, F_GETFL);

	if(opts < 0) Debug(W_MAX, "Erreur lors de fcntl(F_GETFL)");
	else if(fcntl(sock, F_SETFL, opts | O_NONBLOCK) < 0)
		Debug(W_MAX, "Erreur lors de fcntl(F_SETFL)");
	else return;
	exit(EXIT_FAILURE);
}

void w2c_closesocks(void)
{
	int i = 0;

	for(;i <= highsock;++i)	if(FD_ISSET(i, &global_fd_set)) close(i);

	FD_ZERO(&global_fd_set);
	return;
}

void w2c_initsocket(void)
{
	unsigned int reuse_addr = 1;
	struct sockaddr_in localhost; /* bind info structure */

	memset(&localhost, 0, sizeof localhost);
	FD_ZERO(&global_fd_set);

	bind_sock = socket(AF_INET, SOCK_STREAM, 0); /* on demande un socket */
	if(bind_sock < 0)
	{
		Debug(W_MAX|W_TTY, "Impossible de crer le serveur de DB. (socket non cr)");
		exit(EXIT_FAILURE);
	}
	fcntl(bind_sock, F_SETFL, O_NONBLOCK);

	setsockopt(bind_sock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof reuse_addr);

	localhost.sin_family = AF_INET;
	localhost.sin_addr.s_addr = INADDR_ANY;
	localhost.sin_port = htons(bot.w2c_port);

	if(bind(bind_sock, (struct sockaddr *) &localhost, sizeof localhost) < 0)
	{
		Debug(W_MAX|W_TTY, "Impossible d'couter au port %d pour le serveur de DB.", bot.w2c_port);
		close(bind_sock);
		exit(EXIT_FAILURE);
	}

	listen(bind_sock, 5);
	if(bind_sock > highsock) highsock = bind_sock;
	FD_SET(bind_sock, &global_fd_set);

	printf("Serveur de base de donnes du Channel Service OK (Port %d)\n", bot.w2c_port);

	for(reuse_addr = 0;reuse_addr < ASIZE(myWClient);++reuse_addr)
		myWClient[reuse_addr].flag = W2C_FREE;
}

WClient *w2c_addclient(int fd, const char *ip)
{
	WClient *new = NULL;
	unsigned int i = 0;

	DEBUGF(("Adding client with IP %s at %d", ip, fd));

	if(CurrentTS - w2c_lastcheck > W2C_MAXIDLE)
	{
		for(;i < ASIZE(myWClient);++i)
			if(!(myWClient[i].flag & W2C_FREE) && CurrentTS - myWClient[i].lastread > W2C_MAXIDLE)
				w2c_delclient(&myWClient[i]);
		w2c_lastcheck = CurrentTS;
	}

	if((unsigned) fd >= ASIZE(myWClient))
		Debug(W_WARN, "W2C: Trop de clients connects! (Dernier %d sur %s, Max: %d)",
			fd, ip, ASIZE(myWClient));
	else if(!(myWClient[fd].flag & W2C_FREE))
		Debug(W_MAX|W_WARN, "W2C: Connexion sur un slot dj occup!? (%s -> %d[%s])",
			ip, fd, myWClient[fd].ip);
	else
	{	/* register the socket in client list */
		new = &myWClient[fd];
		new->fd = fd;
		if(fd > highsock) highsock = fd;
		FD_SET(fd, &global_fd_set);
		strcpy(new->ip, ip);
		new->user = NULL;
		new->flag = 0;
	}

	return new;
}

void w2c_delclient(WClient *del)
{
	if(del->fd >= highsock) /* Was the highest..*/
		for(;highsock >= bot.sock;--highsock) /* loop only till bot.sock */
			if(!(myWClient[highsock].flag & W2C_FREE)) break;

	if(del->buflen) w2c_dequeue(del); /* flush data if any */
	FD_CLR(del->fd, &global_fd_set);

	DEBUGF(("Exiting client %s from %d", del->ip, del->fd));
	close(del->fd);
	memset(del, 0, sizeof *del); /* clean up */
	del->flag = W2C_FREE; /* now available for a new con */
	return;
}

static int w2c_findcmd(const char *cmd)
{
	unsigned int i = 0;
	for(;i < ASIZE(w2c_cmds);++i)
		if(!strcasecmp(w2c_cmds[i].cmd, cmd)) return i;
	return -1;
}

static int w2c_parsemsg(WClient *cl)
{
	char *parv[MAXPARAM + 1];
	register char *ptr;
	int cmd, parc = 1;

	DEBUGF(("Parsing [%s] from client %s@%d", cl->RecvBuf, cl->ip, cl->fd));

	if((!IsAuth(cl) && strncasecmp(cl->RecvBuf, "pass ", 5))
		|| (IsAuth(cl) && !cl->user && strncasecmp(cl->RecvBuf, "login ", 6)
			&& strncasecmp(cl->RecvBuf, "register ", 9)))
			return w2c_exit_client(cl, "erreur Non autoris");

	parv[0] = cl->RecvBuf; /* cmd */
	if(!(ptr = strchr(cl->RecvBuf, ' ')))
		return w2c_exit_client(cl, "erreur Syntaxe incorrecte");

	*ptr++ = 0;

	if((cmd = w2c_findcmd(parv[0])) < 0)
		return w2c_exit_client(cl, "erreur Commande inconnue");

	while(*ptr) /* split buf in a char * array */
	{
		while(*ptr == ' ') *ptr++ = 0; /* drop spaces */
		if(!*ptr || parc >= MAXPARAM) break;
		else parv[parc++] = ptr;
		while(*ptr && *ptr != ' ') ++ptr; /* go through token till end or space */
	}
	parv[parc] = NULL; /* ends array with NULL */

	if(parc <= w2c_cmds[cmd].args) return w2c_exit_client(cl, "erreur Syntaxe incorrecte");
	w2c_cmds[cmd].func(cl, parc, parv);

	return 0;
}

int w2c_handle_read(int fd)
{
	WClient *cl;
	char buf[400];
	int read;

	if((unsigned) fd >= ASIZE(myWClient))
		return Debug(W_MAX|W_WARN, "Web: reading data from sock #%d, not registered ?", fd);

	cl = &myWClient[fd];

	if((read = recv(fd, buf, sizeof buf -1, 0)) <= 0)
	{
		Debug(W_MAX|W_WARN, "Web: Erreur lors de recv(%s): [%s] (%d)",
			cl->ip, strerror(errno), errno);
		w2c_delclient(cl);
	}
	else
	{
		char *ptr = buf;

		buf[read] = 0;
		bot.WEBtrafficDL += read;

		/* split read buffer in lines,
		   if its length is more than acceptable, truncate it.
		   if a newline is found is the trailing buffer, go on parsing,
		   otherwise abort..
		   if line is unfinished, store it in fd own recv's buffer (cl->RecvBuf)
		  */
		while(*ptr)
		{
			if(*ptr == '\n' || cl->recvlen >= W2C_RECVSIZE)
			{
				cl->RecvBuf[cl->recvlen-1] = 0;
				cl->lastread = CurrentTS;
				w2c_parsemsg(cl);
				if(cl->flag & W2C_FREE) break; /* in case of failed pass */

				if(cl->recvlen >= W2C_RECVSIZE && !(ptr = strchr(ptr + 1, '\n')))
				{ 	/* line exceeds size and no newline found */
					cl->recvlen = 0;
					break; /* abort parsing */
				}
				cl->recvlen = 0; /* go on on newline */
			}
			else cl->RecvBuf[cl->recvlen++] = *ptr; /* copy */
			/*next char, Note that if line was to long but newline was found, it drops the \n */
			++ptr;
		}
	}
	return 0;
}

#endif
